!/usr/bin/env bash

echo -e "[DEBUG] Starting CI trigger to run a sample app of nntrainer actually."
source ../taos/common/api_collection.sh
source /etc/environment
# The 'wget' command saves a log message with the "-o logfile" option while downloading files.
# Run a locale setting which supports 'utf-8' to avoid an issue  that some file names
# in the log file are broken.

export LC_ALL=en_US.UTF-8
export LANG=en_US.UTF-8
export LANGUAGE=en_US:en
export PATH="/var/www/html/.local/bin:$PATH"

REFERENCE_REPOSITORY=/var/www/html/TAOS-CI
app_test_result="pass"

echo "[DEBUG] locale information: start ---------------------------------------------"
locale
echo "[DEBUG] locale information: end   ---------------------------------------------"

echo -e "#######################################################################"
echo -e "[MODULE] ${BOT_NAME}/pr-postbuild-nnstreamer-apptest: Starting a sample app test"
echo -e "[DEBUG] Checking dependencies of required command..."
check_cmd_dep meson
check_cmd_dep ninja
check_cmd_dep wget
check_cmd_dep python
check_cmd_dep Xvnc
check_cmd_dep git
check_cmd_dep insmod
check_cmd_dep cat
check_cmd_dep grep
check_cmd_dep usermod
check_cmd_dep xauth
check_cmd_dep touch
check_cmd_dep awk
check_cmd_dep grep
check_cmd_dep ps
check_cmd_dep hotdoc

# ########## Step 1/6: Set-up environment variables.
export NNTRAINER_ROOT=${1}
# export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$NNST_ROOT/lib
# export GST_PLUGIN_PATH=$GST_PLUGIN_PATH:$NNST_ROOT/lib/gstreamer-1.0
# echo -e "[DEBUG] NNST_ROOT is '$NNST_ROOT'"
# echo -e "[DEBUG] LD_LIBRARY_PATH is '$LD_LIBRARY_PATH'"
# echo -e "[DEBUG] GST_PLUGIN_PATH is '$GST_PLUGIN_PATH'"

# # nnstreamer env variables and include paths
# export NNSTREAMER_CONF=$NNST_ROOT/nnstreamer.ini
# export C_INCLUDE_PATH=$C_INCLUDE_PATH:$NNST_ROOT/include
# export CPLUS_INCLUDE_PATH=$CPLUS_INCLUDE_PATH:$NNST_ROOT/include

declare -i result=0

pushd ${NNTRAINER_ROOT}

########## Step 2/6: Build and install source codes.
# Put a timer in front of the build job to check a start time.
time_start=$(date +"%s")

# Check if a 'build' folder already exists.
if [[ -d ./build ]]; then
    rm -rf ./build/*
else
    mkdir build
fi

# Build a source code of the nnstreamer repository
### Add -Denable-doc=true After merging hotdoc PR
# meson --prefix=${NNTRAINER_ROOT} --sysconfdir=${NNTRAINER_ROOT} --libdir=lib --bindir=bin --includedir=include build

# Install a nnstreamer library
# ninja -C build install

# # Clone the nnstreamer-example repository
# git clone https://github.com/nnsuite/nnstreamer-example.git example-tmp

# # Build and install nnstreamer examples
# cd example-tmp
# meson --prefix=${NNST_ROOT} --sysconfdir=${NNST_ROOT} --libdir=lib --bindir=bin --includedir=include build
# ninja -C build install
# cd ..

# # After installation, binary files are installed to 'bin' folder.
# rm -rf example-tmp
# rm -rf build
# cd bin

# ########## Step 3/6: Download a model and label file for Tensorflow-lite.
# bash get-model-image-classification-tflite.sh
# result=$?

# # Check if the files are normally downloaded
# if [[ ${result} -ne 0 ]]; then
#     echo -e "[DEBUG][FAILED] Oooops!!!!!! apptest is failed."
#     echo -e "[DEBUG][FAILED] The data files was not downloaded. Please check the log file to get a hint"
#     echo -e ""

#     message="Oooops. apptest is failed. Resubmit the PR after fixing correctly.."
    
#     return ${result}
# fi

# ########## Step 4/6: Install a fake USB camera device

# # Run a Xvnc service with a port number 6031 in order to avoid a conflict possibility
# # with existing Xvnc service ports(6000 ~ 6030),
# declare -i xvnc_port=31

# # Prepare a fake (=virtual) USB camera to run video applications.
# # Use '-e' option instead of '-f' because /dev/video0 file is a device file (without a normal file).
# if [[ ! -e /dev/video0 ]]; then
#     echo -e "[DEBUG] An USB Camera device is not enabled. It is required by {nnstreamer_example_image_classification_tflite|nnstreamer_example_cam}."
#     echo -e "[DEBUG] Enabling virtual cam camera..."

#     # Install a 'v4l2loopback' kernel module to use a virtual camera device
#     # if it is not installed.
#     if [[ ! -d ${REFERENCE_REPOSITORY}/v4l2loopback ]]; then
#         echo -e "[DEBUG] Install virtual camera device..."
#         pushd ${REFERENCE_REPOSITORY}
#         git clone https://github.com/umlaeute/v4l2loopback.git
#         popd
#     fi

#     # Go to '${REFERENCE_REPOSITORY}/v4l2loopback' directory
#     pushd ${REFERENCE_REPOSITORY}/v4l2loopback
#     make clean && make

#     # Load kernel modules to run a virtual camera device (e.g., /dev/video0).
#     # The module dependencies: (1) media.ko --> (2) videodev.ko --> (3) v4l2loopback.ko
#     sudo insmod /lib/modules/`uname -r`/kernel/drivers/media/media.ko
#     sudo insmod /lib/modules/`uname -r`/kernel/drivers/media/v4l2-core/videodev.ko
#     sudo insmod ./v4l2loopback.ko

#     # Note that the server administrator must add a 'www-data' (for Apache)
#     # into the 'video' group of /etc/group in order that 'www-data' accesses /dev/video*.
#     # Please, do not specify a '777' permission to avoid a security vulnerability.
#     echo -e "[DEBUG] The group 'video' has to include 'www-data' in the 'video' group."
#     cat /etc/group | grep video

#     # Leave '${REFERENCE_REPOSITORY}/v4l2loopback' directory
#     popd

#     # Establish the file newly unless ~/.Xauthority exists
#     xauth_file=".Xauthority"
#     if [[ -f ~/${xauth_file} ]]; then
#         echo -e "[DEBUG][PASS] It's okay. ~/${xauth_file} exists."
#     else
#         echo -e "[DEBUG][FAIL] It's failed. We can not find ~/${xauth_file}."
#         echo -e "[DEBUG] Initializing ~/${xauth_file} newly ..."
#         touch ~/${xauth_file}
#     fi


#     # Kill the Xvnc service if (1)/dev/video0 file does not exist and (2)port 6031 is still opened.
#     declare -i xvnc_pid=0
#     xvnc_pid=$(ps -A -o pid,cmd | grep "[\:]${xvnc_port}" | awk '{printf $1}')
#     if [[ $xvnc_pid -ne 0 ]]; then
#         echo -e "[DEBUG] It seems that the network port of the existing Xvnc is not stopped."
#         echo -e "[DEBUG] Killing the existing Xvnc (PID: $xvnc_pid) ...."
#         kill $xvnc_pid
#     fi

#     # The VNCserver listens on three ports: 5800 (for VNCweb), 5900 (for VNC), and 6000 (for Xvnc)
#     Xvnc :${xvnc_port} &
#     export DISPLAY=0.0:${xvnc_port}
# else
#     echo -e "[DEBUG] An USB Camera device is enabled.."
# fi

# ########## Step 5/6: Test sample applications (Based on a producer and consumer model)

# # The test scenario to evaluate a stability of applications, is as following:
# #  a. Start a producer
# #  b. Start consumer(s)
# #     - Connect the producer to open a session for virtual display environment.
# #     - Run an application while 2 seconds arbitrarily
# #     - Kill a process of the application after 2 seconds. Otherwise, the process runs forever.
# #  c. Report the execution result with a webhook API.

# # App (Producer): Make a producer with a 'videotestsrc' plugin and /dev/video0 (fake USB camera)
# # The dependency: /dev/video0, VNC
# export DISPLAY=0.0:${xvnc_port}
# declare -i producer_id=0

# echo -e "[DEBUG] ------------------------------------------------------------------"
# echo -e "[DEBUG] App (Producer): Starting 'gst-launch-1.0 videotestsrc ! v4l2sink device=/dev/video0' test on the VNC environment..."
# gst-launch-1.0 -v -e videotestsrc ! v4l2sink device=/dev/video0 &
# producer_id=$!

# if [[ $producer_id -ne 0 ]]; then
#     echo -e "[DEBUG] It's okay. The producer (pid: ${producer_id}) is successfully started."
# else
#     echo -e "[DEBUG] It's failed. The producer (pid: ${producer_id}) is not established."
# fi

# # Display a current locale setting.
# echo -e "[DEBUG] -------------------- locale: start --------------------"
# locale
# echo -e "[DEBUG] -------------------- locale: end --------------------"

# # Display a port status to check a port that VNC and Xvnc has opened.
# echo -e "[DEBUG] -------------------- netstat(Xvnc:59xx): start --------------------"
# netstat -natp | grep [^]]:59
# echo -e "[DEBUG] -------------------- netstat(Xvnc:59xx): end   --------------------"
# echo -e "[DEBUG] -------------------- netstat(Xvnc:60xx): start --------------------"
# netstat -natp | grep [^]]:60
# echo -e "[DEBUG] -------------------- netstat(Xvnc:60xx): end   --------------------"

# # Display an environment setting status that is set by Apache/www-data.
# echo -e "[DEBUG] -------------------- env: start ------------------------"
# env
# echo -e "[DEBUG] -------------------- env: end   ------------------------"

# # Display a listing of the .Xauthority file, enter the following.
# echo -e "[DEBUG] -------------------- xauth: start ----------------------"
# xauth list
# echo -e "[DEBUG] -------------------- xauth: end   ----------------------"


# # App (Consumer 1): Test /dev/video0 status with gst-lanch-1.0 command
# # The dependency: /dev/video0, VNC
# echo -e ""
# echo -e "[DEBUG] ------------------------------------------------------------------"
# echo -e "[DEBUG] App (Consumer 1): Starting 'gst-launch-1.0 v4l2src device=/dev/video0 ! videoconvert ! ximagesink' test on the Xvnc environment..."
# gst-launch-1.0 -v -e v4l2src device=/dev/video0 ! videoconvert ! ximagesink &
# pid=$!
# sleep 2
# kill ${pid}
# if [[ $? -ne 0 ]]; then
#     echo "[DEBUG][FAILED] app test: Oooops!! App (Consumer 1) is failed!"
#     app_test_result="fail"
# else
#     echo "[DEBUG][PASSED] app test: App (Consumer 1) passed"
# fi

# # App (Consumer 2): ./nnstreamer_example_image_classification_tflite for a video image classification.
# # The dependency: /dev/video0, VNC
# echo -e ""
# echo -e "[DEBUG] ------------------------------------------------------------------"
# echo -e "[DEBUG] App (Consumer 2): Starting nnstreamer_example_image_classification_tflite test..."
# ./nnstreamer_example_image_classification_tflite &
# pid=$!
# sleep 2
# kill ${pid}
# if [[ $? -ne 0 ]]; then
#     echo "[DEBUG][FAILED] app test: Oooops!! App (Consumer 2) is failed!"
#     app_test_result="fail"
# else
#     echo "[DEBUG][PASSED] app test: App (Consumer 2) passed"
# fi


# # App (Consumer 3): ./nnstreamer_example_image_classification_tflite.py for a video image classification.
# # Same as above. The difference is that it just runs with python.
# # The dependency: /dev/video0, VNC
# echo -e ""
# echo -e "[DEBUG] ------------------------------------------------------------------"
# echo -e "[DEBUG] App (Consumer 3): Starting nnstreamer_example_image_classification_tflite.py test..."
# python nnstreamer_example_image_classification_tflite.py &
# pid=$!
# sleep 2
# kill ${pid}
# if [[ $? -ne 0 ]]; then
#     echo "[DEBUG][FAILED] app test: Oooops!! App (Consumer 3) is failed!"
#     app_test_result="fail"
# else
#     echo "[DEBUG][PASSED] app test: App (Consumer 3) passed"
# fi

# # App (Consumer 4): ./nnstreamer_example_cam to test a video mixer with nnstreamer plug-in.
# # The dependency: /dev/video0, VNC
# echo -e ""
# echo -e "[DEBUG] ------------------------------------------------------------------"
# echo -e "[DEBUG] App (Consumer 4): Starting nnstreamer_example_cam test..."
# ./nnstreamer_example_cam &
# pid=$!
# sleep 2
# kill ${pid}
# if [[ $? -ne 0 ]]; then
#     echo "[DEBUG][FAILED] app test: Oooops!! App (Consumer 4) is failed!"
#     app_test_result="fail"
# else
#     echo "[DEBUG][PASSED] app test: App (Consumer 4) passed"
# fi

# # App (Consumer 5): ./nnstreamer_sink_example to convert video images to tensor.
# # The dependency: Nothing
# echo -e ""
# echo -e "[DEBUG] ------------------------------------------------------------------"
# echo -e "[DEBUG] App (Consumer 5): Starting nnstreamer_sink_example test..."
# ./nnstreamer_sink_example
# if [[ $? -ne 0 ]]; then
#     echo "[DEBUG][FAILED] app test: Oooops!! App (Consumer 1) is failed!"
#     app_test_result="fail"
# else
#     echo "[DEBUG][PASSED] app test: App (Consumer 1) passed"
# fi

# # App (Consumer 6): ./nnstreamer_sink_example_play to convert video images to tensor,
# # tensor buffer pass another pipeline, and convert tensor to video images.
# # The dependency: VNC
# echo -e ""
# echo -e "[DEBUG] ------------------------------------------------------------------"
# echo -e "[DEBUG] App (Consumer 6): Starting nnstreamer_sink_example_play test..."
# ./nnstreamer_sink_example_play &
# pid=$!
# sleep 2
# kill ${pid}
# if [[ $? -ne 0 ]]; then
#     echo "[DEBUG][FAILED] app test: Oooops!! App (Consumer 1) is failed!"
#     app_test_result="fail"
# else
#     echo "[DEBUG][PASSED] app test: App (Consumer 1) passed"
# fi

# # Let's stop the existing producer ID when all test applications (=consumers) are tested.
# kill ${producer_id}

# popd

# # Put a timer behind the build job to check an end time.
# time_end=$(date +"%s")
# time_diff=$(($time_end-$time_start))
# time_build_cost="$(($time_diff / 60))m $(($time_diff % 60))s"

# ########## Step 6/6: Report a execution result

# # Summarize a test result before doing final report.
# if [[ "$app_test_result" == "fail" ]]; then
#     echo -e "[DEBUG][FAILED] Oooops!!!!!! apptest is failed after $time_build_cost."
#     exit 1
# else
#     echo -e "[DEBUG][PASSED] Successfully apptest is passed in $time_build_cost."
#     exit 0
# fi

